<?php

/**
 * Implements shortcode for linklay
 */
class linklay extends shortcode {

  public function __construct() {
    parent::__construct('linklay');
  }

  /**
   * Returns the markup for the iframe and wrapper elements
   * @param misc $atts Shortcode attributes
   * @param string $content The Linklay ID
   * @return string The HTML displayed
   */
// example: linklay59fff9229e4fe6.97620297
  function do_shortcode($atts, $content = null) {
    extract($atts, EXTR_PREFIX_ALL, 'atts');
    $image = "https://images.linklay.com/" . $content;
    $linklay = "https://assets.linklay.com/embed_cloud.html?uuid_hash=" . $content;
    $base_url = "https://linklay.com/app/";
    $script_src = "https://assets.linklay.com/responsive.js";
    $imgwidth = $imgheight = 0;
    $text_align = null;
    if (!empty($atts_align)) {
      switch (strtolower($atts_align)) {
        case 'center':
          $text_align = 'text-align:center;';
          break;
        case 'right':
          $text_align = 'text-align:right;';
          break;
        case 'left':
          $text_align = 'text-align:left;';
          break;
      }
    }
    $class = !empty($atts_class) ? 'class="' . $atts_class . '" ' : '';
    $markup = <<<EOD
      <div {$class}style="{$text_align}margin:0px; padding:0px; flex: 1 1 0%; position:relative;">
        <iframe width="{$imgwidth}" height="{$imgheight}" style="overflow:hidden;margin-top:-0px;" src="{$linklay}&base_url={$base_url}" allowtransparency="true" frameborder="0">
        </iframe>
        <script src="{$script_src}" onerror="javascript:var el = this.parentElement; el.innerHTML=''; var imgEl = document.createElement( 'img' ); imgEl.src='{$image}'; imgEl.style.width='100%'; imgEl.style.maxWidth='{$imgwidth}px'; el.appendChild( imgEl );window.dispatchEvent(new Event('resize'));" >
        </script>
      </div>
EOD;
    return $markup;
  }

}

/**
 * Enables the use of shortcodes through an abstract class
 */
abstract class shortcode {

  var $type = null;

  function __construct($type) {
    $this->type = trim((string) $type);
  }

  public function replace(&$text, $params = []) {
    $regex = "/\[(.*?)\]/";
    preg_match_all($regex, $text, $matches);
    for ($i = 0; $i < count($matches[0]); $i += 2) {
      $startTag = $matches[0][$i];
      $endTag = $matches[0][$i + 1];
      if ($endTag === "[/$this->type]") {
        $content = strstr(substr($text, strpos($text, $startTag) + strlen($startTag)), $endTag, true);
        $literal = $startTag . $content . $endTag;
        preg_match('/ .*(.*?)"/', $startTag, $prm, PREG_OFFSET_CAPTURE);
        $args = explode(' ', $prm[0][0]);
        $result = [];
        foreach ($args as $arg) {
          if (false !== strpos($arg, '=')) {
            $tmp = explode('=', $arg);
            $result[$tmp[0]] = trim($tmp[1], '"');
          }
        }
        $atts = self::shortcodeAtts($params, $result);
        $shortcode[] = ['literal' => $literal, 'content' => $content, 'atts' => $atts];
      }
      //replace shortcodes in content		
      for ($j = 0; $j < count($shortcode); $j++) {
        $value = $this->do_shortcode($shortcode[$j]['atts'], $shortcode[$j]['content']);
        $text = str_replace($shortcode[$j]['literal'], $value, $text);
      }
    }
    return $this;
  }

  protected function shortcodeAtts($pairs, $atts) {
    $out = array();
    foreach ($pairs as $name => $default) {
      if (array_key_exists($name, (array) $atts)) {
        $out[$name] = $atts[$name];
      } else {
        $out[$name] = $default;
      }
    }
    return $out;
  }

  abstract function do_shortcode($atts, $content = null);
}